/*

                GlM[En[xX^

			Copyrights 2010,  Yoshi Hiroshi

*/

#include <avr/io.h>
#include "adxl245.h"

//==========================================================
void adxl245Init()
{
    unsigned char work;
    ADXL245_POWER_ON;
    ADXL245_SCL_FREE;
    ADXL245_SDA_FREE;
    work = 0x06;
    adxl245Write(0x2C, &work, 1);
    work = 0x08;
    adxl245Write(0x2D, &work, 1);
//    work = 0x5F;    
//    adxl245Write(0x38, &work, 1);   // FIFO mode, INT samples = 31
}

//==========================================================
void adxl245SendStartCondition()
{
	ADXL245_SCL_FREE;			// free SCL line
	while( ADXL245_SCL_CURR_VAL == 0 );
	ADXL245_SDA_SET_LOW;		// Initiate SDA line
	ADXL245_SCL_SET_LOW;		// Initiate SCL line
}

//==========================================================
void adxl245SendStopCondition()
{
	ADXL245_SDA_SET_LOW;
	ADXL245_SCL_FREE;
	while( ADXL245_SCL_CURR_VAL == 0 );
	ADXL245_SDA_FREE;
}

//==========================================================
unsigned char adxl245WriteData(unsigned char val)
{
	unsigned char result;
	for(unsigned char count = 0; count < 8; count++)
    {
        if(val & 0x80) {
        	ADXL245_SDA_FREE;			// send 1
        } else {
        	ADXL245_SDA_SET_LOW;	    // send 0
        }
		ADXL245_SCL_FREE;               // toggle SCL to high(free)
		ADXL245_SCL_SET_LOW;            // toggle SCL to low
        val = val << 1;
	}
    ADXL245_SDA_FREE;
    ADXL245_SCL_FREE;                   // toggle SCL to high(free)
    result = ADXL245_SDA_CURR_VAL; 
	ADXL245_SCL_SET_LOW;                // toggle SCL to low
	return (result == 0);
}

//==========================================================
unsigned char adxl245ReadData(unsigned char ack)
{
	unsigned char result = 0;
	for(unsigned char count = 0; count < 8; count++)
    {
		ADXL245_SCL_FREE;               // toggle SCL to high(free)
        result = result << 1;
        result |= ADXL245_SDA_CURR_VAL? 1:0;
		ADXL245_SCL_SET_LOW;            // toggle SCL to low
	}
    if(ack) {
        ADXL245_SDA_FREE;               // send ack bit
    } else {
        ADXL245_SDA_SET_LOW;            // send ack bit
    }
    ADXL245_SCL_FREE;                   // toggle SCL to high(free)
	ADXL245_SCL_SET_LOW;                // toggle SCL to low
    ADXL245_SDA_FREE;                   // release data bit
	return result;
}

//==========================================================
void adxl245Write(unsigned char reg, unsigned char* data, char len)
{
    unsigned char slaveAdrs = ADXL245_ADDRESS_BASE;
	adxl245SendStartCondition();
	adxl245WriteData(slaveAdrs);
    adxl245WriteData(reg);
    while(len > 0) {
        adxl245WriteData(*data ++);
        len--;
    }
	adxl245SendStopCondition();
}

//==========================================================
void adxl245Read(unsigned char reg, unsigned char* data, char len)
{
    unsigned char slaveAdrs = ADXL245_ADDRESS_BASE;
	adxl245SendStartCondition();
	adxl245WriteData(slaveAdrs);
    adxl245WriteData(reg);
    adxl245SendStartCondition();
    adxl245WriteData(slaveAdrs | 0x01);
    while(len > 0) {
        *data ++ = adxl245ReadData(len == 1);
        len--;
    }
	adxl245SendStopCondition();
}

