/*

                GlM[En[xX^

			Copyrights 2010,  Yoshi Hiroshi

*/

#include <avr/io.h>
#include "harbesta.h"
#include "eeprom.h"

//------------------------------
void eepromInit()
{
    EEPROM_POWER_ON;
    EEPROM_SCL_FREE;
    EEPROM_SDA_FREE;
    eepromSendStartCondition();
    for(char index = 0; index < 9; index++)
    {
        EEPROM_SCL_SET_LOW;
        EEPROM_SCL_FREE;
    }
    EEPROM_SCL_SET_LOW;
    eepromSendStartCondition();
    eepromSendStopCondition();
}

//------------------------------
void eepromSendStartCondition()
{
	EEPROM_SCL_FREE;			// free SCL line
	//while( EEPROM_SCL_CURR_VAL == 0 );
	EEPROM_SDA_SET_LOW;		// Initiate SDA line
	EEPROM_SCL_SET_LOW;		// Initiate SCL line
}

//------------------------------
void eepromSendStopCondition()
{
	EEPROM_SDA_SET_LOW;
	EEPROM_SCL_FREE;
	//while( EEPROM_SCL_CURR_VAL == 0 );
	EEPROM_SDA_FREE;
}

//------------------------------
unsigned char eepromWriteData(unsigned char val)
{
	unsigned char result;
	for(unsigned char count = 0; count < 8; count++)
    {
        if(val & 0x80) {
        	EEPROM_SDA_FREE;			// send 1
        } else {
        	EEPROM_SDA_SET_LOW;	    // send 0
        }
		EEPROM_SCL_FREE;               // toggle SCL to high(free)
		EEPROM_SCL_SET_LOW;            // toggle SCL to low
        val = val << 1;
	}
    EEPROM_SDA_FREE;
    EEPROM_SCL_FREE;               // toggle SCL to high(free)
    result = EEPROM_SDA_CURR_VAL;
	EEPROM_SCL_SET_LOW;            // toggle SCL to low
	return (result == 0);
}

//------------------------------
void eepromWriteBlock(int block, unsigned char* data, int len)
{
    if(IS_EEPROM_READY) {
        if(len <= 0) len = EEPROM_BLOCK_SIZE;
        unsigned char slaveAdrs = (block >> 7) & 0x0E;
        slaveAdrs |= EEPROM_ADDRESS_BASE;
    	eepromSendStartCondition();
    	eepromWriteData(slaveAdrs);
        eepromWriteData(block & 0xFF);
      	eepromWriteData(0);
        while(len > 0) {
            eepromWriteData(*data++);
            len--;
            if((len & 0x1F) == 0x1F) {
                regMain(10);          // 
                chargeCap();
            }
        }
    	eepromSendStopCondition();
    }
}

//------------------------------
char eepromIsBusy(int block)
{
    char flag = 0;
    if(IS_EEPROM_READY) {
        unsigned char slaveAdrs = (block >> 7) & 0x0E;
        slaveAdrs |= EEPROM_ADDRESS_BASE;
    	eepromSendStartCondition();
        flag = !eepromWriteData(slaveAdrs);
        eepromSendStopCondition();
    }
	return flag;
}

//------------------------------
unsigned char eepromReadData(unsigned char ack)
{
	unsigned char result = 0;
	for(unsigned char count = 0; count < 8; count++)
    {
		EEPROM_SCL_FREE;               // toggle SCL to high(free)
        result = result << 1;
        result |= EEPROM_SDA_CURR_VAL? 1:0;
		EEPROM_SCL_SET_LOW;            // toggle SCL to low
	}
    if(ack) {
        EEPROM_SDA_FREE;               // send ack bit
    } else {
        EEPROM_SDA_SET_LOW;            // send ack bit
    }
    EEPROM_SCL_FREE;               // toggle SCL to high(free)
	EEPROM_SCL_SET_LOW;            // toggle SCL to low
    EEPROM_SDA_FREE;               // release data bit
	return result;
}


//------------------------------
void eepromReadBlock(int block, unsigned char* data, int len)
{
    if(IS_EEPROM_READY) {
        if(len <= 0) len = EEPROM_BLOCK_SIZE;
        unsigned char slaveAdrs = (block >> 7) & 0x0E;
        slaveAdrs |= EEPROM_ADDRESS_BASE;
    	eepromSendStartCondition();
    	eepromWriteData(slaveAdrs);
        eepromWriteData(block & 0xFF);
      	eepromWriteData(0);
        eepromSendStartCondition();
        eepromWriteData(slaveAdrs | 0x01);
        while(len > 0) {
            *data ++ = eepromReadData(len == 1);
            len--;
        }
    	eepromSendStopCondition();
    }
}

