/*
                GlM[En[xX^

			Copyrights 2010,  Yoshi Hiroshi

                FUSE: low:0x42, high:0xD7, extend:0xFF
*/

#include <avr/io.h>
#include <avr/sleep.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <util/delay.h>
#include <stdio.h>
#include <ctype.h>
#include "harbesta.h"
#include "uart.h"
#include "eeprom.h"
#include "adxl245.h"
#include "logging.h"
#include "remote.h"

volatile char currentClock;
volatile char sleepMode;
volatile char timerClock;
volatile int loopLimit;
volatile int logTimer;
volatile int powerupCounter;
volatile char powerUpState;


//--------------------------------------------------
//
//
//
//--------------------------------------------------
char clockControl(char speed)
{
    char result = currentClock;
    if(currentClock != speed) 
    {
        CLKPR = _BV(CLKPCE);
        CLKPR = speed;
        currentClock = speed;
    }
    return result;
}

//--------------------------------------------------
//
//
//
//--------------------------------------------------
void counterEnable(char speed)
{
    ASSR = _BV(AS2);
    timerClock = speed;
    switch(speed) {
    case 0:
        TCCR2B = _BV(CS21) | _BV(CS20); // 0.25S
        break;
    case 1:
        TCCR2B = _BV(CS22) | _BV(CS20); // 1S
        break;
    case 2:
        TCCR2B = _BV(CS22) | _BV(CS21); // 2S
        break;
    case 8:
        TCCR2B = _BV(CS22) | _BV(CS21) | _BV(CS20); // 8S
        break;
    case 11:
        TCCR2B = _BV(CS20); // 7.8mS
        break;
    default:
        TCCR2B = _BV(CS22) | _BV(CS21); // 2S
        break;
    }
    TIMSK2 |= _BV(TOIE2);
    loop_until_bit_is_clear(ASSR, TCR2BUB);
}

//--------------------------------------------------
//
//
//
//--------------------------------------------------
char getGatherFlag()
{
    return IS_GATHER;
}

//--------------------------------------------------
//
//
//
//--------------------------------------------------
void setGatherFlag(char f)
{
    if(f) {
        SET_GATHER;
    } else {
        CLEAR_GATHER;
    }
}


//--------------------------------------------------
//
//
//
//--------------------------------------------------
inline void regMain(char limit)
{
    ADMUX = 0;      // 3.3V micon
    if(limit < 0) {
        if(IS_POWER_INITED) {
            loopLimit = 32;
        } else {
            loopLimit = 8;
        }
    } else {
        loopLimit = limit;
    }
    while((ACSR & _BV(ACO)) != 0) // Vsense < 3.3
    {
        PORTD |= _BV(PD2);      // switch FET
        PORTD &= ~_BV(PD2);
        if (loopLimit-- == 0) break;
    }
}

//--------------------------------------------------
//
//
//
//--------------------------------------------------
inline void chargeCap()
{
    ADMUX = 1;      // Vin
    if(IS_POWER_INITED || IS_REMOTE) {
        loopLimit = 16;
    } else {
        loopLimit = 64;
    }
    while((ACSR & _BV(ACO)) == 0) // Vin > 10V
    {
        PORTD |= _BV(PD3);      // switch FET ON
        PORTD |= _BV(PD3);      // switch FET ON
        PORTD &= ~_BV(PD3);     // OFF
        if (loopLimit-- < 1) break;
    }
}


//--------------------------------------------------
//
//
//
//--------------------------------------------------
void checkRemoteMode()
{
    if((PINB & _BV(PB2)) == 0) {    // REMOTE mode check
        if(!IS_REMOTE) {               // detected
            SET_REMOTE;                 // enter REMOTE mode
            CLOCK_COMMUNICATION;
            powerUpState = 4;
            counterEnable(11);
            OPAMP1_PWR_ON;  // ope-amp power on
            OPAMP2_PWR_ON;  // ope-amp power on
            PRR &= ~_BV(PRADC);
            ACSR = _BV(ACBG);   // BGLɂB(10A)
            ADCSRB = _BV(ACME); // AC̓͂A}`vNToRƂ
            eepromInit();
            adxl245Init();
            initLogger();
            initRemote();
        }
    } else {                        // LOCAL mode
        if(IS_REMOTE) {                // detected
            if(isSendBuffEmpty()) {
                CLOCK_FAST;
                CLEAR_REMOTE;
                SET_GATHER;
                counterEnable(1);
                PRR |= _BV(PRUSART0);
            }
        }
    }
}


//--------------------------------------------------
//
//
//
//--------------------------------------------------
void timerInterrupt()
{
    if(IS_REMOTE) {
        checkCommand();
        cyclicCommand();
    }
    logTimer++;
    if(logTimer >= (IS_REMOTE ? 128 : 2)) {    // 2sec
        logTimer = 0;
        gatherAdxlData(IS_REMOTE, IS_GATHER);
        if(IS_GATHER) {
            regMain(5);
        }
    }
}

//--------------------------------------------------
//
//
//
//--------------------------------------------------
ISR(TIMER2_OVF_vect)
{
    SET_TIMER_INT;
}

//--------------------------------------------------
//
//
//
//--------------------------------------------------
ISR(USART_UDRE_vect)
{
    SET_UDRE_INT;
}

//--------------------------------------------------
//
//
//
//--------------------------------------------------
void startingUp()
{
    if(powerUpState < 4) powerupCounter++;
    switch(powerUpState) {
    case 0:
        if(powerupCounter >= 3) {
            powerUpState = 1;
            powerupCounter = 0;
            counterEnable(1);
            regMain(5);
        }
        break;
    case 1:
        ADMUX = 2;      // Vc1
        if(powerupCounter >= 15) {
            if((ACSR & _BV(ACO)) == 0) {
                //adxl245Init();
                powerUpState = 2;
                powerupCounter = 0;
PORTC |= _BV(PC4);
            }
        }
        break;
    case 2:
        ADMUX = 3;      // Vc2
        if(powerupCounter > 15) {
            if((ACSR & _BV(ACO)) == 0) {
PORTC |= _BV(PC5);
                powerUpState = 3;
                powerupCounter = 0;
            }
        }
        break;
    case 3:
        ADMUX = 3;      // Vc2
        if(powerupCounter > 60) {
PORTC &= ~_BV(PC4);
PORTC &= ~_BV(PC5);
            adxl245Init();
            eepromInit();
            initLogger();
            powerUpState = 4;
        }
        break;
    case 4:
        SET_POWER_INITED;
        SET_GATHER;
        powerUpState = 5;
        break;
    case 5:
    default:
        break;
    }
}

//--------------------------------------------------
//
//
//
//--------------------------------------------------
void init()
{
    powerupCounter = 0;
    powerUpState = 0;
    logTimer = 0;

    DDRB = 0x00;
    DDRC = _BV(PC5) | _BV(PC4);
    DDRD = 0xFE;
    PORTB = _BV(PB2);   // REMOTE Connection
    PORTC = 0;
    counterEnable(8);
    PRR = 0xBF;
    CLEAR_POWER_INITED;
    CLEAR_REMOTE;
    CLEAR_GATHER;
    CLEAR_TIMER_INT;
    //OPAMP1_PWR_ON;
    //OPAMP2_PWR_ON;
    //PRR &= ~_BV(PRADC);
    //ACSR = _BV(ACBG);   // BGLɂB(10A)
    //ADCSRB = _BV(ACME); // AC̓͂A}`vNToRƂ
    checkRemoteMode();
}

//--------------------------------------------------
//
//
//
//--------------------------------------------------
int main()
{
    currentClock = -1;
    CLOCK_FAST;
    init();

    while(1)
    {
        if(IS_REMOTE) {
            CLOCK_COMMUNICATION;
            char cmnd;
            uartCheckAndSend();
            if(uartCheckAndRecv()) {
                if((cmnd = getCommand())) {
                    doCommand(cmnd);
                }
            }
        };

        if(IS_TIMER_INT) {
            CLEAR_TIMER_INT;
            checkRemoteMode();
            if(!IS_REMOTE) {
                OPAMP1_PWR_ON;
                if(powerUpState >= 1) OPAMP2_PWR_ON;
                CLOCK_SLOW;         // BG̈҂̂߂ɓx
                PRR &= ~_BV(PRADC);
                ACSR = _BV(ACBG);   // BGLɂB(10A)
                ADCSRB = _BV(ACME); // AC̓͂A}`vNToRƂ
            }
            if(!IS_POWER_INITED) {
                startingUp();
            }
            if(powerUpState != 0) {
                regMain(-1);
            }
            chargeCap();
            if(IS_POWER_INITED || IS_REMOTE) {
                timerInterrupt();
            }
            if(IS_POWER_INITED && IS_REMOTE) {
                //unsigned char work[6];
                //adxl245Read(0x32, work, 6);
                //printf_P(PSTR("X:[%4d]"), work[0] + (work[1] << 8));
                //printf_P(PSTR(" Y:[%4d]"), work[2] + (work[3] << 8));
                //printf_P(PSTR(" Z:[%4d]\n"), work[4] + (work[5] << 8));
            }
        }

        if(!IS_REMOTE) 
        {
            CLOCK_FAST;
            ACSR = 0;           // BG off
            PRR = 0xBF;
            OPAMP1_PWR_OFF;     // ope-amp power off
            OPAMP2_PWR_OFF;     // OPE-AMP Power OFF
        }

        sleepMode = IS_REMOTE ? SLEEP_MODE_IDLE : SLEEP_MODE_PWR_SAVE;
        set_sleep_mode(sleepMode);
        sleep_enable();
        sei();
        sleep_cpu();
        sleep_disable();
    }
}

