/*

                GlM[En[xX^

			Copyrights 2010,  Yoshi Hiroshi

*/

#include <avr/io.h>
#include <stdio.h>
#include "harbesta.h"
#include "eeprom.h"
#include "adxl245.h"
#include "logging.h"

volatile char initedFlag;
volatile unsigned char printGatherFlag;
volatile unsigned char pFlag;
volatile unsigned int  pCount;
volatile unsigned char rFlag;
volatile unsigned char wFlag;
volatile unsigned char lastDataSeq;
volatile int lastBlockSeq;
volatile int lastBlock;
volatile unsigned char skipBlock;
volatile int dataIndex;
unsigned long totalWork[3];
unsigned char adxlWork[6];

struct _logBlock logData[3];
struct _logBlock* buffWrite;
struct _logBlock* buffWork;

//=========================================
void initLogger()
{
    buffWrite = &logData[0];
    buffWork = &logData[2];
    printGatherFlag = 1;
    lastDataSeq = 0;
    skipBlock = 0;
    initedFlag = 0;
    pFlag = 0;
    pCount = 0;
}

//=========================================
int getDataSeq()
{
    return lastDataSeq;
}

//=========================================
int getBlockSeq()
{
    return lastBlockSeq;
}

//=========================================
int getBlock()
{
    return lastBlock;
}

//=========================================
int getBlockLost()
{
    return skipBlock;
}


//=========================================
int getNextSeq(int seq)
{
    seq++;
    if(seq > MAX_SEQ_NUMBER) seq = 0;
    return seq;
}

//=========================================
int getNextBlock(int block)
{
    block++;
    if(block >= MUM_LOG_BLOCK) block = 0;
    return block;
}

//=========================================
struct _logBlock* getBlockPtr()
{
    return buffWork;
}

//=========================================
void initGatherData()
{
//    buffWrite = &logData[0];
//    dataIndex = 0;
}

//=========================================
char getPrintGatherFlag()
{
    return printGatherFlag;
}

//=========================================
void setPrintGatherFlag(char f)
{
    printGatherFlag = f;
}


//=========================================
void gatherAdxlData(char remFlag, char gaFlag)
{
    volatile unsigned char initedNowFlag = initedFlag;
    int blockSave;
    int dataIndexSave;
    unsigned char skipBlockSave = skipBlock;
    unsigned char lastDataSeqSave = lastDataSeq;
    int lastBlockSeqSave = lastBlockSeq;
    totalWork[0] = totalWork[1] = totalWork[2] = 0;

    ADMUX = 3;      // Vc2
//    ADMUX = 2;      // Vc1
    if(((ACSR & _BV(ACO)) == 0) || remFlag) {
        pFlag = 1;
        pCount = 0;
    } else {
        if(pFlag != 0) {
            pFlag = 2;
            pCount++;
            if(pCount >= (100 * NUM_LOG_ELEMENT)) pFlag = 0;
        } else {
            pFlag = 0;
        }
    }
    ADMUX = 2;      // Vc1
    if(((pFlag) || ((ACSR & _BV(ACO)) == 0)) && gaFlag) {
//    if(pFlag) {
        if(gaFlag) {
            rFlag = 'R';
        } else {
            rFlag = 'r';
        }
        if(!initedFlag && pFlag) {
            searchLastBlockSeq();
            initedNowFlag = 1;
        }
    } else {
        rFlag = 'X';
    }
    if(initedFlag) 
    {
        //unsigned char numData;
        //adxl245Read(0x39, &numData, 1);
        //numData &= 0x3F;
        //if(numData != 0) {
        //    for(unsigned char index = 0; index < numData; index++)
        //    {
                adxl245Read(0x32, adxlWork, 6);
                totalWork[0] += adxlWork[0] | (adxlWork[1] << 8);
                totalWork[1] += adxlWork[2] | (adxlWork[3] << 8);
                totalWork[2] += adxlWork[4] | (adxlWork[5] << 8);
        //    }
        //    totalWork[0] /= numData;
        //    totalWork[1] /= numData;
        //    totalWork[2] /= numData;
        //}
        buffWrite->ele[dataIndex].x = totalWork[0];
        buffWrite->ele[dataIndex].y = totalWork[1];
        buffWrite->ele[dataIndex].z = totalWork[2];
        blockSave = lastBlock;
        dataIndexSave = dataIndex;
        dataIndex++;
        if(dataIndex >= NUM_LOG_ELEMENT) {
            writeData(buffWrite, gaFlag);
            dataIndex = 0;
            if(buffWrite == &logData[0]) {
                buffWrite = &logData[1];
            } else {
                buffWrite = &logData[0];
            }
        } else {
            if(pFlag == 0) {
                wFlag = '.';
            } else if(pFlag == 1){
                wFlag = 'p';
            } else if(pFlag == 2){
                wFlag = (pCount / NUM_LOG_ELEMENT) + 0x30;
            } else {
                wFlag = '-';
            }
        }

        if(remFlag && gaFlag && printGatherFlag) {
/*
            printf(" X:[%5d]/Y:[%5d]/Z:[%5d]\n", 
                    (int)totalWork[0],
                    (int)totalWork[1],
                    (int)totalWork[2]);
/*/
            printf("  ---ADXL:%c%c BLK:%d/LOST:%d/SEQ:%d/%d,IX:%d  X:[%5d]/Y:[%5d]/Z:[%5d]\n", 
                    rFlag, wFlag, 
                    blockSave, skipBlockSave, lastBlockSeqSave, lastDataSeqSave, 
                    dataIndexSave,
                    (int)totalWork[0],
                    (int)totalWork[1],
                    (int)totalWork[2]);
 
        }
    }
    initedFlag = initedNowFlag;
}

//=========================================
void writeData(struct _logBlock* buff, char gaFlag)
{
    if(pFlag && gaFlag) {
        if(!initedFlag) {
            searchLastBlockSeq();
            initedFlag = 1;
        }
        wFlag = 'W';
        buff->blockSeq = lastBlockSeq;
        buff->dataSeq = lastDataSeq;
        buff->blockSkip = skipBlock;
        eepromWriteBlock(lastBlock, (unsigned char*)buff, -1);
        lastBlock = getNextBlock(lastBlock);
        lastBlockSeq = getNextSeq(lastBlockSeq);
        skipBlock = 0;
    } else {
        wFlag = 'X';
        if(skipBlock != 0xFF) {
            skipBlock++;
        }
    }
    lastDataSeq++;
}

//=========================================
void searchLastBlockSeq()
{
    lastBlockSeq = -1;
    lastBlock = 0;
    for(int index = 0; index < MUM_LOG_BLOCK; index++)
    {
        eepromReadBlock(index, (unsigned char*)buffWork, 2);
        if(buffWork->blockSeq == 0xFFFF) break;
        if((index > 0) && (buffWork->blockSeq > (lastBlockSeq + 2))) break;
        if(buffWork->blockSeq == 0) lastBlockSeq = 0;
        if(buffWork->blockSeq < lastBlockSeq) lastBlockSeq = buffWork->blockSeq;
        if(buffWork->blockSeq > lastBlockSeq) lastBlockSeq = buffWork->blockSeq;
        lastBlock = getNextBlock(lastBlock);
        if((index & 0x07) == 7) {
            regMain(24);
            chargeCap();
        }
    }
    if(lastBlockSeq != -1) {
        lastBlockSeq = getNextSeq(lastBlockSeq);
    }
    lastBlockSeq = getNextSeq(lastBlockSeq);
}


//=========================================
void eraceBlockData()
{
    buffWork->blockSeq = 0xFFFF;
    buffWork->blockSkip = 0xFF;
    buffWork->dataSeq = 0xFF;
    for(unsigned int index = 0; index < NUM_LOG_ELEMENT; index++)
    {
        buffWork->ele[index].x = 0xFFFF;
        buffWork->ele[index].y = 0xFFFF;
        buffWork->ele[index].z = 0xFFFF;
    }
}

//=========================================
void eraceBlock(int block, char flag, char force)
{
    long work;
    eepromReadBlock(block, (unsigned char*)&work, 4);
    if(flag) eraceBlockData();
    if((work != -1) || force) {
        eepromWriteBlock(block, (unsigned char*)buffWork, -1);
    }
}

//=========================================
void writeTestData(int block, int seq)
{
    buffWork->blockSeq = seq;
    buffWork->dataSeq = seq + block;
    for(int index = 0; index < NUM_LOG_ELEMENT; index++)
    {
        buffWork->ele[index].x = seq * 0x100 + index;
        buffWork->ele[index].y = seq * 0x100 + index + 0x1000;
        buffWork->ele[index].z = seq * 0x100 + index + 0x2000;
    }
    eepromWriteBlock(block, (unsigned char*)buffWork, -1);
}

