/*

                GlM[En[xX^

			Copyrights 2010,  Yoshi Hiroshi

*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <stdio.h>
#include <ctype.h>
#include "harbesta.h"
#include "uart.h"
#include "eeprom.h"
#include "adxl245.h"
#include "logging.h"

volatile char cmndStatus;
volatile char cmndType;
volatile char subType;
volatile char paramSign;
volatile char paramCount;
volatile char subCount;
volatile int param;
volatile int subParam1;
volatile int buffIndex;
volatile int buffCounter;
volatile struct _logBlock* blockPtr;
volatile struct _logEle* buffPtr;


//==========================================================
#define NUM_HELP_MSG        23

char helpMsgLst1[]  PROGMEM = "    List seq number          L\n";
char helpMsgLst2[]  PROGMEM = "    Send all data to PC      S\n";
char helpMsgLst19[] PROGMEM = "    Gather control           G\n";
char helpMsgLst20[] PROGMEM = "    Print gather result      P\n";
char helpMsgLst21[] PROGMEM = "    Read status              Ro\n";
char helpMsgLst22[] PROGMEM = "      options      0:write control\n";
char helpMsgLst23[] PROGMEM = "                   1:eeprom summary\n";
char helpMsgLst3[]  PROGMEM = "    Dump Block by Location   Dnnn\n";
char helpMsgLst4[]  PROGMEM = "    Find Block by Seq. No.   Fnnn\n";
char helpMsgLst5[]  PROGMEM = "    Erace All                E-\n    Erace Specified Block    Ennn\n";
char helpMsgLst6[]  PROGMEM = "    Write test data          W\n";
char helpMsgLst7[]  PROGMEM = "    ADXL245 control\n";
char helpMsgLst8[]  PROGMEM = "      read register          ARxx\n";
char helpMsgLst9[]  PROGMEM = "      write register         AWxx,xx\n";
char helpMsgLst10[] PROGMEM = "      list ADXL register     AL\n";
char helpMsgLst11[] PROGMEM = "        00:DEVID,      1D:THRESH_TAP,   1E:OFSX,       1F:OFSY\n";
char helpMsgLst12[] PROGMEM = "        20:OFSZ,       21:DUR,          22:Latent,     23:Window\n";
char helpMsgLst13[] PROGMEM = "        24:THRESH_ACT, 25:THRESH_INACT, 26:TIME_INACT, 27:ACT_INACT_CTL\n";
char helpMsgLst14[] PROGMEM = "        28:THRESH_FF,  29:TIME_FF,      2A:TAP_AXES,   2B:ACT_TAP_STATUS\n";
char helpMsgLst15[] PROGMEM = "        2C:BW_RATE,    2D:POWER_CTL,    2E:INT_ENABLE, 2F:INT_MAP\n";
char helpMsgLst16[] PROGMEM = "        30:INT_SOURCE, 31:DATA_FORMAT,  32:DATAX0,     33:DATAX1\n";
char helpMsgLst17[] PROGMEM = "        34:DATAY0,     35:DATAY1,       36:DATAZ0,     37:DATAZ1\n";
char helpMsgLst18[] PROGMEM = "        38:FIFO_CTL,   39:FIFO_STATUS\n";

PGM_P helpMsgLst[] = {
    helpMsgLst1,
    helpMsgLst2,
    helpMsgLst19,
    helpMsgLst20,
    helpMsgLst21,
    helpMsgLst22,
    helpMsgLst23,
    helpMsgLst3,
    helpMsgLst4,
    helpMsgLst5,
    helpMsgLst6,
    helpMsgLst7,
    helpMsgLst8,
    helpMsgLst9,
    helpMsgLst10,
    helpMsgLst11,
    helpMsgLst12,
    helpMsgLst13,
    helpMsgLst14,
    helpMsgLst15,
    helpMsgLst16,
    helpMsgLst17,
    helpMsgLst18
};


//==========================================================
void initRemote()
{
    cmndStatus = 0;
// VAʐM
    initUART();
    printf_P(PSTR("Welcome to Harbesta.\n"));
}

//==========================================================
char getCommand()
{
    char result = 0;
    char  errFlag = 1;
    char newValue;
    char newChar = toupper(getchar());
    printf_P(PSTR("%c"), newChar);
    if(iscntrl(newChar)) {
        if((newChar != 0x0D) || (cmndStatus != 2)) {
            printf_P(PSTR("\n"));
            if((cmndType == 'E') || (cmndType == 'W')) {
                while(eepromIsBusy(buffIndex));
                searchLastBlockSeq();
            }
            cmndStatus = 0;
            cmndType = 0;
        } else {
            result = cmndType;
            cmndStatus = 3;
        }
    } else {
        if(cmndStatus == 0) {
            switch(newChar) {
            case 'A':   // ADXL245 control
            case 'D':   // dump block by location
            case 'E':   // erace
            case 'F':   // get block by seq
            case 'R':   // Read status
                cmndStatus++;
            case 'G':   // gather data
            case 'P':   // print gather data
            case 'H':   // help
            case 'L':   // get seq list
            case 'S':   // send all data
            case 'W':   // write test data
                cmndStatus++;
                cmndType = newChar;
                subType = 0;
                param = 0;
                subParam1 = 0;
                paramSign = 0;
                paramCount = 0;
                subCount = 0;
                break;
            default:
                printf_P(PSTR(" !!!IGNORE!!!\n"));
                cmndStatus = 0;
                cmndType = 0;
                break;
            }
        } else if(cmndStatus == 2) {    // get parameter
            if(isxdigit(newChar)) {
                if((paramCount == 0) && (cmndType == 'A')) {
                } else {
                    if(cmndType == 'A') {
                        newValue = newChar - ((newChar > '9') ? ('A' - 10) : '0');
                    } else {
                        newValue = newChar - '0';
                    }
                    if(subCount) {
                        if(cmndType == 'A') {
                            subParam1 = subParam1 * 16 + newValue;
                            errFlag = 0;
                        } else {
                            if(isdigit(newChar)) {
                                subParam1 = subParam1 * 10 + newValue;
                                errFlag = 0;
                            }
                        }
                    } else {
                        if(cmndType == 'A') {
                            param = param * 16 + newValue;
                            errFlag = 0;
                        } else {
                            if(isdigit(newChar)) {
                                param = param * 10 + newValue;
                                errFlag = 0;
                            }
                        }
                    }
                    paramCount++;
                }
            } else if(paramCount == 0) {
                if((cmndType == 'E') && (newChar == '-')) {
                    paramSign = 1;
                    errFlag = 0;
                } else if((cmndType == 'A') && (newChar == 'L')) {
                    subType = 'L';
                    errFlag = 0;
                } else if((cmndType == 'A') && (newChar == 'R')) {
                    subType = 'R';
                    errFlag = 0;
                } else if((cmndType == 'A') && (newChar == 'W')) {
                    subType = 'W';
                    errFlag = 0;
                }
                paramCount++;
            } else if((cmndType == 'A') && (newChar == ',')) {
                paramCount = 1;
                subCount = 1;
                errFlag = 0;
            }
            if(errFlag) {
                printf_P(PSTR(" !!!BAD NUMBER!!!\n"));
                cmndStatus = 0;
                cmndType = 0;
            }
        }
        if(cmndStatus == 1)
        {
            result = cmndType;
            cmndStatus = 3;
        }
    }
    return result;
}

//==========================================================
void dump1()
{
    printf_P(PSTR("DUMP:%d   "), param);
    if(param >= MUM_LOG_BLOCK) {
        printf_P(PSTR("Block number too large.\n"));
        cmndStatus = 0;
    } else {
        blockPtr = getBlockPtr();
        buffIndex = 0;
        while(eepromIsBusy(buffIndex));
        searchLastBlockSeq();
        buffPtr = &blockPtr->ele[0];
        eepromReadBlock(param, (unsigned char*)blockPtr, -1);
        printf_P(PSTR(" BLOCK:[%d]"), param);
        printf_P(PSTR(" BLOCK_LOST:[%d]"), blockPtr->blockSkip);
        printf_P(PSTR(" BLOCK_SEQ:[%d]"), blockPtr->blockSeq);
        printf_P(PSTR(" DATA_SEQ:[%d]\n"), blockPtr->dataSeq);
        cmndStatus = 4;
    }
}

//==========================================================
void dump2()
{
    if(isSendBuffEmpty()) {
        if(!eepromIsBusy(buffIndex)) {
            for(char c = 0; c < 1; c++) {
                for(int index = 0; index < 2; index ++) {
                    if(index != 0) printf_P(PSTR("    "), buffIndex);
                    printf_P(PSTR("Inx:%4d  "), buffIndex);
                    printf_P(PSTR("X:%4.4X[%6d], Y:%4.4X[%6d], Z:%4.4X[%6d]"), 
                            buffPtr->x, 
                            buffPtr->x, 
                            buffPtr->y, 
                            buffPtr->y, 
                            buffPtr->z,
                            buffPtr->z);
                    buffIndex++;
                    buffPtr++;
                    if(buffIndex >= NUM_LOG_ELEMENT) {
                        cmndType = 0;
                        cmndStatus = 0;
                        break;
                    }
                }
                printf("\n");
                if(buffIndex >= NUM_LOG_ELEMENT) {
                    break;
                }
            }
        }
    }
}

//==========================================================
void find1()
{
    int work;
    printf_P(PSTR("FIND:SEQUENCE NUMBER=%d  "), param);
    if(param <= MAX_SEQ_NUMBER) {
        blockPtr = getBlockPtr();
        buffIndex = 0;
        while(eepromIsBusy(buffIndex));
        buffPtr = &blockPtr->ele[0];
        for(int index = 0; index < MUM_LOG_BLOCK; index++)
        {
            if((index % 16) == 0) printf_P(PSTR("."));
            uartCheckAndSend();
            eepromReadBlock(index, (unsigned char*)&work, 2);
            if(work == param) {
                printf_P(PSTR("FOUND at block:%d\n"), index);
                eepromReadBlock(index, (unsigned char*)blockPtr, -1);
                printf_P(PSTR("GET:%d  "), param);
                printf_P(PSTR(" BLOCK:[%d]"), index);
                printf_P(PSTR(" BLOCK_LOST:[%d]"), blockPtr->blockSkip);
                printf_P(PSTR(" BLOCK_SEQ:[%d]"), blockPtr->blockSeq);
                printf_P(PSTR(" DATA_SEQ:[%d]\n"), blockPtr->dataSeq);
                cmndStatus = 4;
                break;
            }
        }
    }
    if(cmndStatus != 4) {
        printf_P(PSTR("INVALID SEQ NNUMBER.\n"));
        cmndStatus = 0;
        cmndType = 0;
    }
}

//==========================================================
void gatherData()
{
    if(getGatherFlag()) {
        setGatherFlag(0);
        printf_P(PSTR("ather data stop.  ON --> OFF\n"));
    } else {
        setGatherFlag(1);
        printf_P(PSTR("ather data start.   OFF --> ON\n"));
        initGatherData();
    }
    cmndStatus = 0;
    cmndType = 0;
}

//==========================================================
void printGatherData()
{
    if(getPrintGatherFlag()) {
        setPrintGatherFlag(0);
        printf_P(PSTR("rint Gather data DISABLED.  ON --> OFF\n"));
    } else {
        setPrintGatherFlag(1);
        printf_P(PSTR("rint Gather data ENABLED.   OFF --> ON\n"));
    }
    cmndStatus = 0;
    cmndType = 0;
}


//==========================================================
void list1()
{
    buffIndex = 0;
    printf_P(PSTR("ist sequence number.   "));
    searchLastBlockSeq();
    printf_P(PSTR(" nextBLK:[%d]"), getBlock());
    printf_P(PSTR(" nextSEQ:[%d]"), getBlockSeq());
    printf_P(PSTR(" DataSeq:[%d]"), getDataSeq());
    printf_P(PSTR(" lostBLK:[%d]"), getBlockLost());
    printf_P(PSTR("\n"));
    cmndStatus = 4;
}

//==========================================================
void list2()
{
    int work;
    
    if(isSendBuffEmpty()) {
        if(!eepromIsBusy(buffIndex)) {
            for(char c = 0; c < 1; c++) {
                buffCounter = 0;
                printf_P(PSTR(" %4d:"), buffIndex);
                while(buffCounter < 20) {
                    eepromReadBlock(buffIndex, (unsigned char*)&work, 2);
                    printf_P(PSTR(" %4d"), work);
                    buffIndex++;
                    buffCounter++;
                    if(buffIndex >= MUM_LOG_BLOCK) {
                        cmndType = 0;
                        cmndStatus = 0;
                        break;
                    }
                }
                printf_P(PSTR("\n"));
                if(buffIndex >= MUM_LOG_BLOCK) {
                    break;
                }
            }
        }
    }
}


//==========================================================
void adxlControl1()
{
    unsigned char work[4];
    cmndStatus = 0;
    if(subType == 'R') {
        adxl245Read(param, &work[0], 1);
        printf_P(PSTR("Read from ADXL register.  %02X(%d):%02X(%d)"), param, param, work[0], work[0]);
    } else if(subType == 'W') {
        work[0] = subParam1;
        adxl245Write(param, &work[0], 1);
        adxl245Read(param, &work[0], 1);
        printf_P(PSTR("Write into ADXL register.  %02X(%d):%02X(%d):%02X(%d)"), param, param, subParam1, subParam1, work[0], work[0]);
    } else if(subType == 'L') {
        printf_P(PSTR("DUMP ADXL register.\n  %02X(%3d):%02X(%3d)"), 0, 0, work[0], work[0]);
        buffIndex = 29;
        cmndStatus = 4;
    }
    printf_P(PSTR("\n"));
}
//==========================================================
void adxlControl2()
{
    unsigned char work[8];
    if(isSendBuffEmpty()) {
        if(subType == 'L') {
            for(unsigned char c = 0; c < 4; c++) {
                adxl245Read(buffIndex, work, 1);
                printf_P(PSTR("   %02X[%3d]:%02X[%3d]"), buffIndex, buffIndex, work[c], work[c]);
                buffIndex++;
                if(buffIndex == 1) buffIndex = 29;
                if(buffIndex == 32) break;
                if(buffIndex > 57) {
                    cmndStatus = 0;
                    break;
                }
            }
            printf_P(PSTR("\n"));
        }
    }
}

//==========================================================
void readStatus1()
{
    printf_P(PSTR("Read Staus.  "));
    if((param == 0) || (param == 1)) {
        searchLastBlockSeq();
        printf_P(PSTR(" nextBLK:[%d]"), getBlock());
        printf_P(PSTR(" nextSEQ:[%d]"), getBlockSeq());
        printf_P(PSTR(" DataSeq:[%d]"), getDataSeq());
        printf_P(PSTR(" lostBLK:[%d]"), getBlockLost());
        printf_P(PSTR("\n"));
        if(getGatherFlag()) {
            printf_P(PSTR("  Gather:[ON]. "));
        } else {
            printf_P(PSTR("  Gather:[OFF]."));
        }
        if(getPrintGatherFlag()) {
            printf_P(PSTR("    Print Gather Result:[ON].\n"));
        } else {
            printf_P(PSTR("    Print Gather Result:[OFF].\n"));
        }
        if(param == 0) {
            cmndStatus = 0;
        } else {
            buffIndex = 0;
            cmndStatus = 4;
        }
    } else {
        printf_P(PSTR("Option error.\n"));
    }
}

void readStatus2()
{
    unsigned char work[4];
    
    if(isSendBuffEmpty()) {
        if(!eepromIsBusy(buffIndex)) {
            for(char c = 0; c < 1; c++) {
                buffCounter = 0;
                while(buffCounter < 4) {
                    printf_P(PSTR("   %4d:"), buffIndex); // BLK No.
                    eepromReadBlock(buffIndex, work, 4);
                    printf_P(PSTR("BS=[%4d]"), work[0] | (work[1] << 8));   // BLK Seq.
                    printf_P(PSTR(",L=[%3d]"), work[2]);   // BLK Lost
                    printf_P(PSTR(",DS=[%3d]"), work[3]);   // Data Seq.
                    buffIndex++;
                    buffCounter++;
                    if(buffIndex >= MUM_LOG_BLOCK) {
                        cmndType = 0;
                        cmndStatus = 0;
                        break;
                    }
                }
                printf_P(PSTR("\n"));
                if(buffIndex >= MUM_LOG_BLOCK) {
                    break;
                }
            }
        }
    }
}

//==========================================================
void send1()
{
    printf_P(PSTR("END ALL DATA\n"));
    printf_P(PSTR("block, block-lost, block-seq, data-seq_data-index, x, y, z\n"));
    blockPtr = getBlockPtr();
    buffPtr = &blockPtr->ele[0];
    buffIndex = 0;
    buffCounter = 0;
    while(eepromIsBusy(buffIndex));
    eepromReadBlock(buffIndex, (unsigned char*)blockPtr, -1);
    if(blockPtr->blockSeq < 0) {
        cmndType = 0;
        cmndStatus = 0;
    } else {
        cmndStatus = 4;
    }
}

//==========================================================
void send2()
{
    if(isSendBuffEmpty()) {
        if(!eepromIsBusy(buffIndex)) {
            for(int index = 0; index < 1; index ++) {
                printf_P(PSTR("%d,"), buffIndex);
                printf_P(PSTR("%d,"), blockPtr->blockSkip);
                printf_P(PSTR("%d,"), blockPtr->blockSeq);
                printf_P(PSTR("%d"), blockPtr->dataSeq);
                printf_P(PSTR("%2.2d,"), buffCounter);
                printf_P(PSTR("%d,"), buffPtr->x);
                printf_P(PSTR("%d,"), buffPtr->y);
                printf_P(PSTR("%d\n"), buffPtr->z);
                buffCounter++;
                buffPtr++;
                if(buffCounter >= NUM_LOG_ELEMENT) {
                    buffIndex++;
                    if(buffIndex < MUM_LOG_BLOCK) {
                        buffPtr = &blockPtr->ele[0];
                        buffCounter = 0;
                        eepromReadBlock(buffIndex, (unsigned char*)blockPtr, -1);
                    } else {
                        cmndType = 0;
                        cmndStatus = 0;
                    }
                    break;
                }
            }
            if(blockPtr->blockSeq < 0) {
                cmndType = 0;
                cmndStatus = 0;
            }
        }
    }
}

//==========================================================
void erase1()
{
    if(paramSign) {
        printf_P(PSTR("-- Erace all.\n"));
        eraceBlock(0, 1, 0);
        buffIndex = 0;
        cmndStatus = 4;
    } else {
        printf_P(PSTR("\nErace block. BLK=%d\n"), param);
        eraceBlock(param, 1, 1);
        while(eepromIsBusy(buffIndex));
        searchLastBlockSeq();
        cmndType = 0;
        cmndStatus = 0;
    }
}

//==========================================================
void erase2()
{
    if(isSendBuffEmpty()) {
        if(!eepromIsBusy(buffIndex)) {
            buffIndex++;
            if(buffIndex >= MUM_LOG_BLOCK) {
                cmndType = 0;
                cmndStatus = 0;
                searchLastBlockSeq();
                printf_P(PSTR("End erace\n"));
            } else {
                if((buffIndex % 16) == 0) printf_P(PSTR("."));
                eraceBlock(buffIndex, 0, 0);
            }
        }
    }
}

//==========================================================
void write1()
{
    printf_P(PSTR("rite test data.\n"));
    buffIndex = 0;
    buffCounter = 21;
    cmndStatus = 4;
}

//==========================================================
void write2()
{
    if(isSendBuffEmpty()) {
        if(!eepromIsBusy(buffIndex)) {
            if((buffIndex % 16) == 0) printf_P(PSTR("."));
            if(buffIndex == 140) buffCounter = 820;
            writeTestData(buffIndex, buffCounter);
            buffIndex++;
            buffCounter = getNextSeq(buffCounter);
            if(buffIndex >= MUM_LOG_BLOCK) {
                cmndType = 0;
                cmndStatus = 0;
                searchLastBlockSeq();
                printf_P(PSTR("\nEnd of data write.  nextSEQ:%d nextBLK:%d\n"), getBlockSeq(), getBlock());
            }
        }
    }
}

//==========================================================
void help1()
{
    printf_P(PSTR("elp. ========== COMMAND LIST =====\n"));
    buffIndex = 0;
    cmndStatus = 4;
}

//==========================================================
void help2()
{
    if(isSendBuffEmpty()) {
        printf_P(helpMsgLst[buffIndex]);
        buffIndex++;
        if(buffIndex >= NUM_HELP_MSG) {
            cmndType = 0;
            cmndStatus = 0;
            printf_P(PSTR("========== END OF COMMAND LIST =====\n"));
        }
    }
}

//==========================================================
void doCommand(char cmnd)
{
    switch(cmnd) {
    case 'A':
        adxlControl1();
        break;
    case 'D':
        dump1();
        break;
    case 'E':
        erase1();
        break;
    case 'F':
        find1();
        break;
    case 'G':
        gatherData();
        break;
    case 'H':
        help1();
        break;
    case 'L':
        list1();
        break;
    case 'P':
        printGatherData();
        break;
    case 'R':
        readStatus1();
        break;
    case 'S':
        send1();
        break;
    case 'W':
        write1();
        break;
    default:
        break;
    }
}

//==========================================================
void checkCommand()
{
    if(cmndStatus != 4) return;
    switch(cmndType) {
    case 'A':
        adxlControl2();
        break;
    case 'D':
    case 'F':
        dump2();
        break;
    case 'E':
        erase2();
        break;
    case 'H':
        help2();
        break;
    case 'L':
        list2();
        break;
    case 'R':
        readStatus2();
        break;
    case 'S':
        send2();
        break;
    case 'W':
        write2();
        break;
    default:
        break;
    }
}

//==========================================================
void cyclicCommand()
{
}
